/*
	PIC18F2550 `USBpUSBWCXeBbNTv
		ver1.00	2010/11/11	(MCHPFSUSB̃Tv쐬)

	o[iX^Yj{6{^ƂĔF܂B
	A0Ƀ{^tAƁAuEv->uv->uEv+{^P3/60bœ͂܂
*/

#ifndef JOYSTICKMAIN_C
#define JOYSTICKMAIN_C

/** INCLUDES *******************************************************/
#include <timers.h>
#include "Microchip/Include/USB/usb.h"
#include "HardwareProfile.h"
#include "Microchip/Include/USB/usb_function_hid.h"
#include "bootload.h"


#define WRITE_TIMER0	0x9E58

/** PRIVATE PROTOTYPES *********************************************/
static void InitializeSystem(void);
void ProcessIO(void);
void UserInit(void);
void Joystick(void);

/** VARIABLES ******************************************************/
#pragma udata
char buffer[8];
USB_HANDLE lastTransmission;
BOOL Keyboard_out;
char command_state = 0;

//	ɃR}hB
//	R}h͌납珇Ɏs
#define COMMAND_MACRO_COUNT	4
BYTE command_macro[COMMAND_MACRO_COUNT][3] = 
	{{0x00,0x80,0x80},	//͂_
	 {0x01,0xff,0xff},
	 {0x00,0x80,0xff},
	 {0x00,0xff,0x80}
	};

#pragma udata USB_VARIABLES=0x500
BYTE joystick_input[3];
BYTE hid_report[8];
#pragma udata


#pragma code
void isr_high()
{
	#if defined(USB_INTERRUPT)
	USBDeviceTasks();
	#endif
}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 
void isr_low()
{
	if(INTCONbits.TMR0IF == 1)
	{
		INTCONbits.TMR0IF = 0;
		WriteTimer0(WRITE_TIMER0);

		if(command_state != 0)
			command_state--;

		joystick_input[0] = command_macro[command_state][0];
		joystick_input[1] = command_macro[command_state][1];
		joystick_input[2] = command_macro[command_state][2];
	}
}	//This return will be a "retfie", since this is in a #pragma interruptlow section 


/********************************************************************
 * Function:        void main(void)
 *******************************************************************/
void main(void)
{   
    InitializeSystem();

    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

    while(1)
    {
        #if defined(USB_POLLING)
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
        #endif
    				  
        ProcessIO();        
    }//end while
}//end main


/********************************************************************
 * Function:        static void InitializeSystem(void)
 *******************************************************************/
static void InitializeSystem(void)
{
	ADCON1 |= 0x0F;                 // Default all pins to digital

    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
    UserInit();

    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
}//end InitializeSystem



/******************************************************************************
 * Function:        void UserInit(void)
 *****************************************************************************/
void UserInit(void)
{
	TRISA = 0x03;	//RA0,1͂
	TRISB = 0;
	LATB = 0;

//	^C}0̐ݒ
//	48MHzA1/60sɊ荞݂𔭐
//	(1/60) / (4 / 48000000) / 8 = 25000
//	65536 - 25000 = 40536 = 0x9E58
	OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_SOURCE_INT & T0_PS_1_8);
	WriteTimer0(WRITE_TIMER0);

//	荞݊Jn
	INTCON2bits.TMR0IP = 0;		//^C}0ʃx荞݂ɐݒ
	INTCONbits.TMR0IE = 1;		//^C}0荞݋
	INTCONbits.GIEL = 1;		//ʃx荞݂̋
	

	joystick_input[0] = 0x00;
	joystick_input[1] = 0x80;
	joystick_input[2] = 0x80;

    lastTransmission = 0;
}//end UserInit


/********************************************************************
 * Function:        void ProcessIO(void)
 *******************************************************************/
void ProcessIO(void)
{   
    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

   Joystick();
}//end ProcessIO


/******************************************************************************
 * Function:        void Joystick(void)
 *****************************************************************************/
void Joystick(void)
{
    if(!HIDTxHandleBusy(lastTransmission))
    {
        if(!PORTAbits.RA0)
        {
	        if(command_state == 0)
				command_state = COMMAND_MACRO_COUNT;
        }
      	lastTransmission = HIDTxPacket(HID_EP, (BYTE*)&joystick_input, sizeof(joystick_input));
    }
    return;		
}//end joystick()


// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *****************************************************************************/
void USBCBSuspend(void)
{
}

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *******************************************************************/
void USBCBErrorHandler(void)
{
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}//end

/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *******************************************************************/
void USBCBInitEP(void)
{
    USBEnableEndpoint(HID_EP,USB_OUT_ENABLED|USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}

/** EOF main.c *************************************************/
#endif
